@@ -62,6 +62,7 @@ DEFAULT_DB_FOSSILS = DB_DIR / "fossils.csv"
 DEFAULT_DB_PRICES = DB_DIR / "prices.csv"
 DEFAULT_DB_ART = DB_DIR / "art.csv"
 DEFAULT_DB_USERS = DB_DIR / "users.csv"
+DEFAULT_DB_FISH = DB_DIR / "fish.csv"
 
 # temporary application files
 TMP_DIR = RUNTIME_ROOT / "tmp"
@@ -94,15 +95,17 @@ USER_PREFRENCES = [
     "creator",
 ]
 
+# Based on values from datetime.isoweekday()
 DAYS = {
-    "sunday": 0,
     "monday": 1,
     "tuesday": 2,
     "wednesday": 3,
     "thursday": 4,
     "friday": 5,
     "saturday": 6,
+    "sunday": 7,
 }
+IDAYS = dict(map(reversed, DAYS.items()))
 
 
 class Validate:
@@ -157,6 +160,13 @@ def h(dt):
     return naturaltime(naive_dt)
 
 
+def day_and_time(dt):
+    """Converts a datetime to a day and time of day, eg: Monday pm."""
+    day = IDAYS[dt.isoweekday()]
+    am_pm = "am" if dt.hour < 12 else "pm"
+    return f"{day.title()} {am_pm}"
+
+
 def humanize_months(row):
     """Generator that humanizes months from row data where each month is a column."""
     ABBR = {
@@ -275,6 +285,7 @@ class Turbot(discord.Client):
         channels=[],
         prices_file=DEFAULT_DB_PRICES,
         art_file=DEFAULT_DB_ART,
+        fish_file=DEFAULT_DB_FISH,
         fossils_file=DEFAULT_DB_FOSSILS,
         users_file=DEFAULT_DB_USERS,
         log_level=None,
@@ -286,11 +297,13 @@ class Turbot(discord.Client):
         self.channels = channels
         self.prices_file = prices_file
         self.art_file = art_file
+        self.fish_file = fish_file
         self.fossils_file = fossils_file
         self.users_file = users_file
         self.base_prophet_url = "https://turnipprophet.io/?prices="  # TODO: configurable?
         self._prices_data = None  # do not use directly, load it from load_prices()
         self._art_data = None  # do not use directly, load it from load_art()
+        self._fish_data = None  # do not use directly, load it from load_fish()
         self._fossils_data = None  # do not use directly, load it from load_fossils()
         self._users_data = None  # do not use directly, load it from load_users()
         self._last_backup_filename = None
@@ -379,6 +392,20 @@ class Turbot(discord.Client):
                 self._art_data = pd.DataFrame(columns=["author", "name"])
         return self._art_data
 
+    def save_fish(self, data):
+        """Saves the given fish data to csv file."""
+        data.to_csv(self.fish_file, index=False)  # persist to disk
+        self._fish_data = data  # in-memory optimization
+
+    def load_fish(self):
+        """Returns a DataFrame of fish data or creates an empty one."""
+        if self._fish_data is None:
+            try:
+                self._fish_data = pd.read_csv(self.fish_file)
+            except FileNotFoundError:
+                self._fish_data = pd.DataFrame(columns=["author", "name"])
+        return self._fish_data
+
     def save_fossils(self, data):
         """Saves the given fossils data to csv file."""
         data.to_csv(self.fossils_file, index=False)  # persist to disk
@@ -568,7 +595,9 @@ class Turbot(discord.Client):
             return [None] * 13
 
         buy_date = recent_buy.timestamp.iloc[0]
-        if buy_date.to_pydatetime().isoweekday() != 7:  # buy isn't on a sunday
+        if (
+            buy_date.to_pydatetime().isoweekday() != DAYS["sunday"]
+        ):  # buy isn't on a sunday
             return [None] * 13
 
         buy_price = int(recent_buy.price.iloc[0])
@@ -588,7 +617,7 @@ class Turbot(discord.Client):
         groups = sells.set_index("timestamp").groupby(pd.Grouper(freq="D"))
         for day, df in groups:
             day_of_week = day.to_pydatetime().isoweekday()
-            if day_of_week == 7:  # no sells allowed on sundays
+            if day_of_week == DAYS["sunday"]:  # no sells allowed on sundays
                 continue
             for ts, row in df.iterrows():
                 if ts.hour < 12:  # am
@@ -684,15 +713,21 @@ class Turbot(discord.Client):
                     for i, page in enumerate(pages):
                         file = (
                             attachment
-                            if attachment
+                            if attachment is not None
                             and i == last_page_index
                             and n == last_reply_index
                             else None
                         )
                         await message.channel.send(page, file=file)
                 elif isinstance(reply, discord.embeds.Embed):
-                    file = attachment if attachment and n == last_reply_index else None
+                    file = (
+                        attachment
+                        if attachment is not None and n == last_reply_index
+                        else None
+                    )
                     await message.channel.send(embed=reply, file=file)
+                else:
+                    raise RuntimeError("non-string non-embed reply not supported")
 
     ##############################
     # Discord Client Behavior
@@ -792,7 +827,7 @@ class Turbot(discord.Client):
         now = self.to_usertime(user_id, datetime.now(pytz.utc))
         start = now - timedelta(days=now.isoweekday() % 7)  # start of week
         start = datetime(start.year, start.month, start.day, tzinfo=start.tzinfo)
-        day_offset = DAYS[day_of_week]
+        day_offset = DAYS[day_of_week] % 7
         hour_offset = 13 if time_of_day == "evening" else 0
         return start + timedelta(days=day_offset, hours=hour_offset)
 
@@ -913,11 +948,13 @@ class Turbot(discord.Client):
         yours = prices[prices.author == target_id]
         lines = [s("history_header", name=target_name)]
         for _, row in yours.iterrows():
+            time = self.to_usertime(target_id, row.timestamp)
             lines.append(
                 s(
                     f"history_{row.kind}",
                     price=row.price,
-                    timestamp=h(self.to_usertime(target_id, row.timestamp)),
+                    timestamp=h(time),
+                    day_and_time=day_and_time(time),
                 )
             )
         return "\n".join(lines), None
@@ -1018,7 +1055,21 @@ class Turbot(discord.Client):
             lines.append(s("collect_bugs"))
 
         if valid_fish:
-            lines.append(s("collect_fish"))
+            fish = self.load_fish()
+            yours = fish[fish.author == author.id]
+            dupes = yours.loc[yours.name.isin(valid_fish)].name.values.tolist()
+            new_names = list(set(valid_fish) - set(dupes))
+            new_data = [[author.id, name] for name in new_names]
+            new_fish = pd.DataFrame(columns=fish.columns, data=new_data)
+            fish = fish.append(new_fish, ignore_index=True)
+            yours = fish[fish.author == author.id]  # re-fetch for congrats
+            self.save_fish(fish)
+            if new_names:
+                lines.append(s("collect_fish_new", items=", ".join(sorted(new_names))))
+            if dupes:
+                lines.append(s("collect_fish_dupe", items=", ".join(sorted(dupes))))
+            if len(FISH_SET) == len(yours.index):
+                lines.append(s("congrats_all_fish"))
 
         if valid_art:
             art = self.load_art()
@@ -1034,7 +1085,7 @@ class Turbot(discord.Client):
                 lines.append(s("collect_art_new", items=", ".join(sorted(new_names))))
             if dupes:
                 lines.append(s("collect_art_dupe", items=", ".join(sorted(dupes))))
-            if len(ART) == len(yours.index):
+            if len(ART_SET) == len(yours.index):
                 lines.append(s("congrats_all_art"))
 
         if invalid:
@@ -1082,7 +1133,21 @@ class Turbot(discord.Client):
             lines.append(s("uncollect_bugs"))
 
         if valid_fish:
-            lines.append(s("uncollect_fish"))
+            fish = self.load_fish()
+            yours = fish[fish.author == author.id]
+            previously_collected = yours.loc[yours.name.isin(valid_fish)]
+            deleted = set(previously_collected.name.values.tolist())
+            didnt_have = valid_fish - deleted
+            fish = fish.drop(previously_collected.index)
+            self.save_fish(fish)
+            if deleted:
+                lines.append(
+                    s("uncollect_fish_deleted", items=", ".join(sorted(deleted)))
+                )
+            if didnt_have:
+                lines.append(
+                    s("uncollect_fish_already", items=", ".join(sorted(didnt_have)))
+                )
 
         if valid_art:
             art = self.load_art()
@@ -1125,28 +1190,35 @@ class Turbot(discord.Client):
         fossils = self.load_fossils()
         fossil_users = fossils.author.unique()
         fossil_results = defaultdict(list)
-        for fossil in valid_fossils:
-            havers = fossils[fossils.name == fossil].author.unique()
+        for collected_fossil in valid_fossils:
+            havers = fossils[fossils.name == collected_fossil].author.unique()
             needers = np.setdiff1d(fossil_users, havers).tolist()
             for needer in needers:
                 name = discord_user_from_id(channel, needer)
-                fossil_results[name].append(fossil)
+                fossil_results[name].append(collected_fossil)
 
         if valid_bugs:
             return s("search_bugs"), None
 
-        if valid_fish:
-            return s("search_fish"), None
+        fish = self.load_fish()
+        fish_users = fish.author.unique()
+        fish_results = defaultdict(list)
+        for collected_fish in valid_fish:
+            havers = fish[fish.name == collected_fish].author.unique()
+            needers = np.setdiff1d(fish_users, havers).tolist()
+            for needer in needers:
+                name = discord_user_from_id(channel, needer)
+                fish_results[name].append(collected_fish)
 
         art = self.load_art()
         art_users = art.author.unique()
         art_results = defaultdict(list)
-        for artpiece in valid_art:
-            havers = art[art.name == artpiece].author.unique()
+        for collected_art in valid_art:
+            havers = art[art.name == collected_art].author.unique()
             needers = np.setdiff1d(art_users, havers).tolist()
             for needer in needers:
                 name = discord_user_from_id(channel, needer)
-                art_results[name].append(artpiece)
+                art_results[name].append(collected_art)
 
         if not fossil_results and not art_results and not invalid:
             return s("search_all_not_needed"), None
@@ -1155,6 +1227,8 @@ class Turbot(discord.Client):
         needed = set()
         for items in fossil_results.values():
             needed.update(items)
+        for items in fish_results.values():
+            needed.update(items)
         for items in art_results.values():
             needed.update(items)
         not_needed = searched - needed
@@ -1163,6 +1237,9 @@ class Turbot(discord.Client):
         for name, items in fossil_results.items():
             items_str = ", ".join(sorted(items))
             lines.append(s("search_fossil_row", name=name, items=items_str))
+        for name, items in fish_results.items():
+            items_str = ", ".join(sorted(items))
+            lines.append(s("search_fish_row", name=name, items=items_str))
         for name, items in art_results.items():
             items_str = ", ".join(sorted(items))
             lines.append(s("search_art_row", name=name, items=items_str))
@@ -1197,6 +1274,11 @@ class Turbot(discord.Client):
         collected_fossils = set(your_fossils.name.unique())
         remaining_fossils = FOSSILS_SET - collected_fossils
 
+        fish = self.load_fish()
+        your_fish = fish[fish.author == target_id]
+        collected_fish = set(your_fish.name.unique())
+        remaining_fish = FISH_SET - collected_fish
+
         art = self.load_art()
         your_art = art[art.author == target_id]
         collected_art = set(your_art.name.unique())
@@ -1221,6 +1303,16 @@ class Turbot(discord.Client):
         else:
             lines.append(s("congrats_all_fossils"))
 
+        if remaining_fish:
+            lines.append(
+                s("uncollected_fish_count", count=len(remaining_fish), name=target_name)
+            )
+            lines.append(
+                s("uncollected_fish_remaining", items=", ".join(sorted(remaining_fish)))
+            )
+        else:
+            lines.append(s("congrats_all_fish"))
+
         if remaining_art:
             lines.append(
                 s("uncollected_art_count", count=len(remaining_art), name=target_name)
@@ -1278,18 +1370,25 @@ class Turbot(discord.Client):
         collected_fossils = set(your_fossils.name.unique())
         all_fossils = len(collected_fossils) == len(FOSSILS_SET)
 
+        fish = self.load_fish()
+        your_fish = fish[fish.author == target_id]
+        collected_fish = set(your_fish.name.unique())
+        all_fish = len(collected_fish) == len(FISH_SET)
+
         art = self.load_art()
         your_art = art[art.author == target_id]
         collected_art = set(your_art.name.unique())
         all_art = len(collected_art) == len(ART_SET)
 
         lines = []
-        if any([all_fossils, all_art]):
+        if any([all_fossils, all_fish, all_art]):
             if all_fossils:
                 lines.append(s("congrats_all_fossils"))
+            if all_fish:
+                lines.append(s("congrats_all_fish"))
             if all_art:
                 lines.append(s("congrats_all_art"))
-            if all([all_fossils, all_art]):
+            if all([all_fossils, all_fish, all_art]):
                 return "\n".join(lines), None
 
         if collected_art and not all_art:
@@ -1301,6 +1400,17 @@ class Turbot(discord.Client):
                     items=", ".join(sorted(collected_art)),
                 )
             )
+
+        if collected_fish and not all_fish:
+            lines.append(
+                s(
+                    "collected_fish",
+                    name=target_name,
+                    count=len(collected_fish),
+                    items=", ".join(sorted(collected_fish)),
+                )
+            )
+
         if collected_fossils and not all_fossils:
             lines.append(
                 s(
@@ -1388,13 +1498,20 @@ class Turbot(discord.Client):
                     s("count_fossil_valid", name=user_name, count=len(remaining))
                 )
 
+            lines.append(s("count_fish_valid_header"))
+            fish = self.load_fish()
+            for user_name, user_id in sorted(valid):
+                yours = fish[fish.author == user_id]
+                collected = set(yours.name.unique())
+                remaining = FISH_SET - collected
+                lines.append(s("count_fish_valid", name=user_name, count=len(remaining)))
+
             lines.append(s("count_art_valid_header"))
             art = self.load_art()
             for user_name, user_id in sorted(valid):
                 yours = art[art.author == user_id]
                 collected = set(yours.name.unique())
-                allnames = set(ART.name.unique())
-                remaining = allnames - collected
+                remaining = ART_SET - collected
                 lines.append(s("count_art_valid", name=user_name, count=len(remaining)))
 
         if invalid:
@@ -1449,6 +1566,55 @@ class Turbot(discord.Client):
 
         return response, None
 
+    def creatures_available_now(self, now, df):
+        """Returns the names of creatures in df that are available right now."""
+        for _, row in df.iterrows():
+            time = row["time"]
+            if time.lower() == "all day":
+                yield row["name"]
+            else:
+                ranges = time.lower().split("&")
+                for r in ranges:
+                    lhs, rhs = [t.strip().lower() for t in r.split("-")]
+                    lhs_hour, lhs_ampm = lhs.split(" ")
+                    rhs_hour, rhs_ampm = rhs.split(" ")
+                    lhs_hour, rhs_hour = int(lhs_hour), int(rhs_hour)
+                    now_hour = now.hour
+                    now_ampm = "am" if now_hour < 12 else "pm"
+
+                    # | lhs_ampm | rhs_ampm | now_ampm | yield if (use _hour vars) |
+                    # | -------- | -------- | -------- | ------------------------- |
+                    # | am       | am       | am       | lhs <= now <= rhs         |
+                    # | am       | am       | pm       | False                     |
+                    # | am       | pm       | am       | lhs <= now <= rhs + 12    |
+                    # | am       | pm       | pm       | lhs <= now <= rhs + 12    |
+                    # | pm       | am       | am       | now <= rhs                |
+                    # | pm       | am       | pm       | now >= lhs                |
+                    # | pm       | pm       | am       | False                     |
+                    # | pm       | pm       | pm       | lhs <= now <= rhs         |
+                    if lhs_ampm == "am" and rhs_ampm == "am" and now_ampm == "am":
+                        if lhs_hour <= now_hour <= rhs_hour:
+                            yield row["name"]
+                    elif lhs_ampm == "am" and rhs_ampm == "am" and now_ampm == "pm":
+                        continue
+                    elif lhs_ampm == "am" and rhs_ampm == "pm" and now_ampm == "am":
+                        if lhs_hour <= now_hour <= rhs_hour + 12:
+                            yield row["name"]
+                    elif lhs_ampm == "am" and rhs_ampm == "pm" and now_ampm == "pm":
+                        if lhs_hour <= now_hour <= rhs_hour + 12:
+                            yield row["name"]
+                    elif lhs_ampm == "pm" and rhs_ampm == "am" and now_ampm == "am":
+                        if now_hour <= rhs_hour:
+                            yield row["name"]
+                    elif lhs_ampm == "pm" and rhs_ampm == "am" and now_ampm == "pm":
+                        if now_hour >= lhs_hour:
+                            yield row["name"]
+                    elif lhs_ampm == "pm" and rhs_ampm == "pm" and now_ampm == "am":
+                        continue
+                    else:  # lhs_ampm == "pm" and rhs_ampm == "pm" and now_ampm == "pm"
+                        if lhs_hour <= now_hour <= rhs_hour:
+                            yield row["name"]
+
     def _creatures(self, *_, author, params, kind, source, force_text=False):
         """The fish and bugs commands are so similar; I factored them out to a helper."""
         hemisphere = self.get_user_prefs(author.id).get("hemisphere", None)
@@ -1478,13 +1644,6 @@ class Turbot(discord.Client):
                 "alert": alert,
             }
 
-        def add_header(lines):
-            if kind != "bugs":
-                return lines
-            if random.randint(0, 100) > 70:
-                lines.insert(0, s("bugs_header"))
-            return lines
-
         if params:
             user_input = " ".join(params)
             search = user_input.lower()
@@ -1497,32 +1656,78 @@ class Turbot(discord.Client):
 
             if found.empty:
                 return s(f"{kind}_none_found", search=user_input)
-            elif force_text or len(found) > EMBED_LIMIT:
-                available = found  # fallback to the less detailed, text only, response
-            else:
-                response = []
-                rows = [row for _, row in found.iterrows()]
-                for row in sorted(rows, key=lambda r: r["name"]):
-                    info = details(row)
-                    embed = discord.Embed(title=info["name"])
-                    embed.set_thumbnail(url=info["image"])
-                    embed.add_field(name="price", value=info["price"])
-                    embed.add_field(name="location", value=info["location"])
-                    if "shadow" in info:
-                        embed.add_field(name="shadow size", value=info["shadow"])
-                    embed.add_field(name="available", value=info["time"])
-                    embed.add_field(name="during", value=info["months"])
-                    if info["alert"]:
-                        embed.add_field(name="alert", value=info["alert"])
-                        if "GONE" in info["alert"]:
-                            embed.color = discord.Color.orange()
-                        else:
-                            embed.color = discord.Color.blue()
-                    response.append(embed)
-                return add_header(response)
+        else:
+            if kind == "fish":
+                caught = self.load_fish()
+                caught = caught[caught.author == author.id]
+            else:  # kind == "bugs"
+                caught = None  # not supported yet
 
-        lines = [s(kind, **details(row)) for _, row in available.iterrows()]
-        return "\n".join(add_header(sorted(lines)))
+            if caught is not None and not caught.empty:
+                already = available.loc[available.name.isin(caught.name)]
+                available = available.drop(already.index)
+
+            found = available
+
+            if found.empty:
+                return s(f"{kind}_none_available")
+
+        now_names = set(self.creatures_available_now(now, found))
+        found_now = found[found.name.isin(now_names)]
+        found_this_month = found[~found.name.isin(now_names)]
+
+        def get_response(df, force_text):
+            if force_text or len(df) > EMBED_LIMIT:
+                lines = [s(kind, **details(row)) for _, row in df.iterrows()]
+                return "\n".join(sorted(lines))
+
+            response = []
+            rows = [row for _, row in df.iterrows()]
+            for row in sorted(rows, key=lambda r: r["name"]):
+                info = details(row)
+                embed = discord.Embed(title=info["name"])
+                embed.set_thumbnail(url=info["image"])
+                embed.add_field(name="price", value=info["price"])
+                embed.add_field(name="location", value=info["location"])
+                if "shadow" in info:
+                    embed.add_field(name="shadow size", value=info["shadow"])
+                embed.add_field(name="available", value=info["time"])
+                embed.add_field(name="during", value=info["months"])
+                if info["alert"]:
+                    embed.add_field(name="alert", value=info["alert"])
+                    if "GONE" in info["alert"]:
+                        embed.color = discord.Color.orange()
+                    else:
+                        embed.color = discord.Color.blue()
+                response.append(embed)
+            return response
+
+        response_now = get_response(found_now, force_text=force_text)
+        response_this_month = get_response(found_this_month, force_text=True)
+
+        responses = []
+
+        if response_now:
+            header = s("creature_available_now", kind=kind.title())
+            if isinstance(response_now, str):
+                responses.append(f"{header}\n{response_now}")
+            else:
+                responses.append(header)
+                responses.extend(response_now)
+
+        if response_this_month:
+            # response_this_month is always forced to be text
+            header = s("creature_available_this_month", kind=kind.title())
+            responses.append(f"{header}\n{response_this_month}")
+
+        def add_header(responses):
+            if kind != "bugs":
+                return responses
+            if random.randint(0, 100) > 70:
+                responses.insert(0, s("bugs_header"))
+            return responses
+
+        return add_header(responses)
 
     @command
     def fish(self, channel, author, params):
@@ -1553,14 +1758,14 @@ class Turbot(discord.Client):
         """
         return (
             [
-                self._creatures(
+                *self._creatures(
                     author=author,
                     params=["arriving"],
                     kind="bugs",
                     source=BUGS,
                     force_text=True,
                 ),
-                self._creatures(
+                *self._creatures(
                     author=author,
                     params=["arriving"],
                     kind="fish",
@@ -1724,22 +1929,18 @@ def get_channels(channels_file):  # pragma: no cover
     help="read authorized channel names from this file",
 )
 @click.option(
-    "-p",
-    "--prices-file",
-    default=DEFAULT_DB_PRICES,
-    help="read price data from this file",
+    "--prices-file", default=DEFAULT_DB_PRICES, help="read price data from this file",
 )
 @click.option(
-    "-a", "--art-file", default=DEFAULT_DB_ART, help="read art data from this file",
+    "--art-file", default=DEFAULT_DB_ART, help="read art data from this file",
 )
 @click.option(
-    "-f",
-    "--fossils-file",
-    default=DEFAULT_DB_FOSSILS,
-    help="read fossil data from this file",
+    "--fish-file", default=DEFAULT_DB_FISH, help="read fish data from this file",
+)
+@click.option(
+    "--fossils-file", default=DEFAULT_DB_FOSSILS, help="read fossil data from this file",
 )
 @click.option(
-    "-u",
     "--users-file",
     default=DEFAULT_DB_USERS,
     help="read users preferences data from this file",
@@ -1759,6 +1960,7 @@ def main(
     auth_channels_file,
     prices_file,
     art_file,
+    fish_file,
     fossils_file,
     users_file,
     dev,
@@ -1783,6 +1985,7 @@ def main(
         channels=auth_channels,
         prices_file=prices_file,
         art_file=art_file,
+        fish_file=fish_file,
         fossils_file=fossils_file,
         users_file=users_file,
         log_level=getattr(logging, "DEBUG" if verbose else log_level),
